#include "stdafx.h"
#include <fstream>
#include "parameters.h"
#include <math.h>

using namespace std;



void load_signed_distance_function(char filename_txt_format[], float signed_dist_array[],parameters &param, short zoom_level){

	long s3D;

	if(zoom_level == 1){
		s3D = param.size3Dl;
	}
	if(zoom_level == 2){
		s3D = param.size3Dl_z2;
	}
	if(zoom_level == 3){
		s3D = param.size3Dl_z3;
	}

	ifstream in_stream;

	in_stream.open(filename_txt_format);
	if (in_stream.fail()){
		cout << "Input file opening failed" << endl;
		exit(1);
	};

	for(long n=0;n<(s3D);n++){
		in_stream >> signed_dist_array[n];
	};

	in_stream.close();




}




float signed_dist(float x, float y, float z, float signed_dist_array[],parameters &param, short zoom_level){

	long s1Dx, s1Dy, s1Dz;

	if(zoom_level == 1){
		s1Dx = param.size1D_xl;
		s1Dy = param.size1D_yl;
		s1Dz = param.size1D_zl;
		x=x+1.5;
		y=y+1.5;
		z=z+1.5;
	}
	if(zoom_level == 2){
		s1Dx = param.size1D_xl_z2;
		s1Dy = param.size1D_yl_z2;
		s1Dz = param.size1D_zl_z2;
		x=x+1.0;
		y=y+1.0;
		z=z+1.0;
		x=x-8; x=x/2.0; x=x+8.0;
		y=y-8; y=y/2.0; y=y+8.0;
		z=z-8; z=z/2.0; z=z+8.0;
		
	}
	if(zoom_level == 3){
		s1Dx = param.size1D_xl_z3;
		s1Dy = param.size1D_yl_z3;
		s1Dz = param.size1D_zl_z3;
		x=x-8; x=x/4.0; x=x+8.0;
		y=y-8; y=y/4.0; y=y+8.0;
		z=z-8; z=z/4.0; z=z+8.0;
	}


	x = x*2;
	y = y*2;
	z = z*2;

	float val;
	float sd[8];
	long around[8];


	//note the defalut value is sensible only for tetrahedra refinements with conditions to be met at all points

	if( (x < 0) || (x > (s1Dx-1)) || (y < 0) || (y > (s1Dy-1)) || (z < 0) || (z > (s1Dz-1)) ){
		val = 800;
	}else{

	//finding 8 values around point

		around[0] = floor(x) + floor(y)*s1Dx + floor(z)*s1Dx*s1Dy;
		around[1] = ceil(x) + floor(y)*s1Dx + floor(z)*s1Dx*s1Dy;
		around[2] = floor(x) + ceil(y)*s1Dx + floor(z)*s1Dx*s1Dy;
		around[3] = ceil(x) + ceil(y)*s1Dx + floor(z)*s1Dx*s1Dy;
		around[4] = floor(x) + floor(y)*s1Dx + ceil(z)*s1Dx*s1Dy;
		around[5] = ceil(x) + floor(y)*s1Dx + ceil(z)*s1Dx*s1Dy;
		around[6] = floor(x) + ceil(y)*s1Dx + ceil(z)*s1Dx*s1Dy;
		around[7] = ceil(x) + ceil(y)*s1Dx + ceil(z)*s1Dx*s1Dy;

		for(short m=0;m<8;m++){
			sd[m] = signed_dist_array[around[m]];
		};


		
		float d1b = x - float(floor(x));
		float d2b = y - float(floor(y));
		float d3b = z - float(floor(z));
		float d1 = 1.0-d1b;
		float d2 = 1.0-d2b;
		float d3 = 1.0-d3b;

		val = sd[0]*d1*d2*d3 + sd[1]*d1b*d2*d3 + sd[2]*d1*d2b*d3 + sd[3]*d1b*d2b*d3 + sd[4]*d1*d2*d3b + sd[5]*d1b*d2*d3b + sd[6]*d1*d2b*d3b + sd[7]*d1b*d2b*d3b;

		//cout << x << " " << y << " " << z << endl;
		//cout << d1 << " " << d2 << " " << d3 << " " << d1b << " " <<d2b << " " << d3b << endl;
		//cout << sd[0] << " " << sd[1] << " " << sd[2] << " " << sd[3] << " " << sd[4] << " "<< sd[5] <<" "<< sd[6] << " "<< sd[7]<< endl;
		//cout << val << endl;
	};

	return val;



/*
	float val = sqrt( x*x + y*y + z*z) - 10.0; 

	return val;
*/

}



void load_regions_function(char filename_txt_format[],  short regions_array[],parameters &param, short zoom_level){

	long s3D;

	if(zoom_level == 1){
		s3D = param.size3D;
	}
	if(zoom_level == 2){
		s3D = param.size3D_z2;
	}
	if(zoom_level == 3){
		s3D = param.size3D_z3;
	}


	ifstream in_stream;

	in_stream.open(filename_txt_format);
	if (in_stream.fail()){
		cout << "Input file opening failed" << endl;
		exit(1);
	};

	for(long n=0;n<s3D;n++){
		in_stream >> regions_array[n];
	};

	in_stream.close();




}

long round(float val){

	float one = 1.0;
	float rem;
	rem = fmod(val,one);
	if(rem < .5){
		val = val - rem;
	}else{
		val = val - rem + 1;
	}

	return val;

}



 short region(float x, float y, float z,  short regions_array[],parameters &param, short zoom_level){

	long s1Dx, s1Dy, s1Dz;

	if(zoom_level == 1){
		s1Dx = param.size1D_x;
		s1Dy = param.size1D_y;
		s1Dz = param.size1D_z;
		x=x+1.5;
		y=y+1.5;
		z=z+1.5;
	}
	if(zoom_level == 2){
		s1Dx = param.size1D_x_z2;
		s1Dy = param.size1D_y_z2;
		s1Dz = param.size1D_z_z2;
		x=x+1.0;
		y=y+1.0;
		z=z+1.0;
		x=x-8; x=x/2.0; x=x+8.0;
		y=y-8; y=y/2.0; y=y+8.0;
		z=z-8; z=z/2.0; z=z+8.0;
		
	}
	if(zoom_level == 3){
		s1Dx = param.size1D_x_z3;
		s1Dy = param.size1D_y_z3;
		s1Dz = param.size1D_z_z3;
		x=x-8; x=x/4.0; x=x+8.0;
		y=y-8; y=y/4.0; y=y+8.0;
		z=z-8; z=z/4.0; z=z+8.0;
	}




	long r_x, r_y, r_z;
	float rem_x,rem_y,rem_z;
	long index;

	short val;
	short m_selected;

	if( (x < 0) || (x > (s1Dx-1)) || (y < 0) || (y > (s1Dy-1)) || (z < 0) || (z > (s1Dz-1)) ){
		val = 0;
	}else{



		r_x = round(x);
		r_y = round(y);
		r_z = round(z);

		index = r_x + s1Dx*r_y + s1Dx*s1Dy*r_z;

		val = regions_array[index];

		rem_x = r_x - x;
		rem_y = r_y - y;
		rem_z = r_z - z;

	//finding 8 values around point

		if( (rem_x == .5) || (rem_y == .5) || (rem_z == .5) ){

			long around[8];
			short r[8];

			around[0] = floor(x) + floor(y)*s1Dx + floor(z)*s1Dx*s1Dy;
			around[1] = ceil(x) + floor(y)*s1Dx + floor(z)*s1Dx*s1Dy;
			around[2] = floor(x) + ceil(y)*s1Dx + floor(z)*s1Dx*s1Dy;
			around[3] = ceil(x) + ceil(y)*s1Dx + floor(z)*s1Dx*s1Dy;
			around[4] = floor(x) + floor(y)*s1Dx + ceil(z)*s1Dx*s1Dy;
			around[5] = ceil(x) + floor(y)*s1Dx + ceil(z)*s1Dx*s1Dy;
			around[6] = floor(x) + ceil(y)*s1Dx + ceil(z)*s1Dx*s1Dy;
			around[7] = ceil(x) + ceil(y)*s1Dx + ceil(z)*s1Dx*s1Dy;
			
			for(short m=0;m<8;m++){
				r[m] = regions_array[around[m]];
			}

			if( (rem_x)==.5){
				if( (rem_y <= .5) && (rem_z <= .5) ){
					if(r[0]!=r[1]){
						val = -1;
					}
				}
				if( (rem_y >= .5) && (rem_z <= .5) ){
					if(r[2]!=r[3]){
						val = -1;
					}
				}
				if( (rem_y <= .5) && (rem_z >= .5) ){
					if(r[4]!=r[5]){
						val = -1;
					}
				}
				if( (rem_y >= .5) && (rem_z >= .5) ){
					if(r[6]!=r[7]){
						val = -1;
					}
				}
			}
			if( (rem_y)==.5){
				if( (rem_x <= .5) && (rem_z <= .5) ){
					if(r[0]!=r[2]){
						val = -1;
					}
				}
				if( (rem_x >= .5) && (rem_z <= .5) ){
					if(r[1]!=r[3]){
						val = -1;
					}
				}
				if( (rem_x <= .5) && (rem_z >= .5) ){
					if(r[4]!=r[6]){
						val = -1;
					}
				}
				if( (rem_x >= .5) && (rem_z >= .5) ){
					if(r[5]!=r[7]){
						val = -1;
					}
				}
			}
			if( (rem_z)==.5){
				if( (rem_x <= .5) && (rem_y <= .5) ){
					if(r[0]!=r[4]){
						val = -1;
					}
				}
				if( (rem_x >= .5) && (rem_y <= .5) ){
					if(r[1]!=r[5]){
						val = -1;
					}
				}
				if( (rem_x <= .5) && (rem_y >= .5) ){
					if(r[2]!=r[6]){
						val = -1;
					}
				}
				if( (rem_x >= .5) && (rem_y >= .5) ){
					if(r[3]!=r[7]){
						val = -1;
					}
				}
			}	


		}

		


/*


	//picking the maximimum region number to be the assigned region

		m_selected = 0;
		val = regions_array[around[0]];
		for(short m=1;m<8;m++){
			if(regions_array[around[m]]>regions_array[around[m-1]]){
				val = regions_array[around[m]];
				m_selected = m;
			};
		};
	


//		if( (x==0) || (x==(param.size1D_x-1)) || (y==0) || (y==(param.size1D_y-1)) || (z==0) || (z==(param.size1D_z-1)) ){
//			if( val != 0){
//				val = -1;
//			}
//		}else{


			float one = 1.0;
			long a;
			a = around[m_selected];

			if( fmod(x,one) == 0){

				if( (regions_array[a] != regions_array[a-1]) || (regions_array[a] != regions_array[a+1]) ){
					val = -1;
				}
			}

			if( fmod(y,one) == 0){
				if( (regions_array[a] != regions_array[a-1*param.size1D_x]) || (regions_array[a] != regions_array[a+1*param.size1D_x]) ){
					val = -1;
				}
			}

			if( fmod(z,one) == 0){
				if( (regions_array[a] != regions_array[a-1*param.size1D_x*param.size1D_y]) || (regions_array[a] != regions_array[a+1*param.size1D_x*param.size1D_y]) ){
					val = -1;
				}
			}
*/
//		}

	}

	return val;

}


float loc_integral(float x, float y, float z, float signed_dist_array[],parameters &param){

	float val=0;

	if( (x < 3) || (x >= (param.size1D_x-3)) || (y < 3) || (y >= (param.size1D_y-3)) || (z < 3) || (z >= (param.size1D_z-3)) ){
		val = 200;
	}else{

	//takes the integral of the signed distance function over the 4by4by4 box around the point

	long x_in = floor(x);
	long y_in = floor(y);
	long z_in = floor(z);
	long index;

	
	for(short i=-1;i<3;i++){
		for(short j=-1;j<3;j++){
			for(short k=-1;k<3;k++){

				index = (x_in+i) + (y_in+j)*param.size1D_x + (z_in+k)*param.size1D_x*param.size1D_y;
				val = val + signed_dist_array[index];


			};
		};
	};

	val = val/(64);

	};

	return val;


}












 void pre_process_regions(short regions_array[],parameters &param){

	 for(long n=0;n<param.size3D;n++){
		 if(regions_array[n]<0){
			 regions_array[n] = regions_array[n] + 300;
		 }
	 }
 }




